#!/bin/bash
DB_USER="root"
DB_PASSWORD="<mysql db password>" #set mysql db password
TIME_OUT=120 #modify it if required

PREPOST_LOG="/tmp/prepost.log"
DB_LOCK_FILE="/tmp/mysql_read_lock"

JOB_ID=$JOB_ID

function debug {
	echo "`date +'%Y/%m/%d %H:%M:%S'` [$JOB_ID] $1" >>$PREPOST_LOG
}

function kill_children_processes {
	local parent_pid=$1
	local child_pids=`ps -ef | grep -w $parent_pid |grep -v grep | awk '{if('$parent_pid'==$3){print $2}}'`
	for child_pid in ${child_pids}
	do
		pkill -KILL -P $child_pid >/dev/null 2>&1
		debug "stop the child processes of $child_pid"
	done
}

function check_db_password {
	mysql -u$DB_USER -p$DB_PASSWORD -e "exit" >/dev/null 2>&1
	return $?
}

function set_db_readonly { 
(
  echo "flush tables with read lock;"

  touch $DB_LOCK_FILE
  sleep $TIME_OUT
) | mysql -u$DB_USER -p$DB_PASSWORD >/dev/null 2>&1
}

#main
debug "start to lock mysql db..."
USER_ID=`id -u`
if [ "X$USER_ID" != "X0" ]; then
	debug  "root user is required to lock the db"
	exit 1
fi

check_db_password
if [ $? -ne 0 ]; then
	debug "mysql db password is invalid"
	exit 1
fi

set_db_readonly &
DB_LOCK_PROCESS_PID=$!

ELAPSED_TIME=0
while [ ! -f $DB_LOCK_FILE ]
do
	if ! ps -p $DB_LOCK_PROCESS_PID >/dev/null 2>&1; then
		debug "mysql lock process $DB_LOCK_PROCESS_PID does not exist"
		exit 2
	fi
	
 	if [ $ELAPSED_TIME -gt $TIME_OUT ]; then
		kill_children_processes $DB_LOCK_PROCESS_PID
		debug "wait for read lock time out: $TIME_OUT"
		exit 3
	fi

	sleep 1
	ELAPSED_TIME=`expr $ELAPSED_TIME + 1`
done

echo $DB_LOCK_PROCESS_PID > $DB_LOCK_FILE

debug  "lock mysql db successfully"
exit 0
